package com.kaixed.kchat.data.repository

import com.kaixed.kchat.data.local.dao.UserInfoDao
import com.kaixed.kchat.data.local.entity.UserInfo
import com.kaixed.kchat.data.model.request.RegisterRequest
import com.kaixed.kchat.data.model.response.register.Register
import com.kaixed.kchat.network.ApiCall.apiCall
import com.kaixed.kchat.network.RetrofitClient
import com.kaixed.kchat.utils.Constants.MMKV_USER_SESSION
import com.tencent.mmkv.MMKV

/**
 * @Author: kaixed
 * @Date: 2024/11/23 14:15
 */
class UserAuthRepository(private val userInfoDao: UserInfoDao) {

    private val authApiService = RetrofitClient.authApiService

    private val mmkv by lazy { MMKV.mmkvWithID(MMKV_USER_SESSION) }

    /**
     * 注册方法
     * @param registerRequest 注册请求参数
     */
    suspend fun register(registerRequest: RegisterRequest): Result<Register?> {
        return apiCall(
            apiCall = { authApiService.register(registerRequest) },
            errorMessage = "注册成功，但未返回用户数据"
        ).onSuccess { register ->
            register?.let {
                insertUserInfo(register, registerRequest.telephone)
            }
        }
    }

    /**
     * 登录方法
     * @param username 用户名
     * @param password 密码
     */
    suspend fun loginByUsername(username: String, password: String): Result<UserInfo?> {
        val requestParams = mapOf("username" to username, "password" to password)
        return apiCall(
            apiCall = { authApiService.loginByUsername(requestParams) },
            errorMessage = "登录成功，但未返回用户数据"
        ).onSuccess { userInfo ->
            userInfo?.let {
                updateDb(userInfo)
            }
        }
    }

    /**
     * 登录方法
     * @param telephone 手机号
     * @param password 密码
     */
    suspend fun loginByTelephone(telephone: String, password: String): Result<UserInfo?> {
        val requestParams = mapOf("telephone" to telephone, "password" to password)
        return apiCall(
            apiCall = { authApiService.loginByTelephone(requestParams) },
            errorMessage = "登录成功，但未返回用户数据"
        ).onSuccess { userInfo ->
            userInfo?.let {
                updateDb(userInfo)
            }
        }
    }
    
    private suspend fun insertUserInfo(
        register: Register,
        telephone: String
    ) {
        val userInfo = UserInfo(
            username = register.username,
            nickname = register.nickname,
            avatarUrl = "",
            signature = "",
            telephone = telephone
        )
        userInfoDao.saveUserInfo(userInfo)
    }

    private suspend fun updateDb(userInfo: UserInfo) {
        mmkv.apply {
            encode("username", userInfo.username)
            encode("nickname", userInfo.nickname)
            encode("avatarUrl", userInfo.avatarUrl)
        }
        MMKV.defaultMMKV().encode("userLoginStatus", true)
        val existingUser = userInfoDao.getUserInfoByUsername(userInfo.username)

        if (existingUser == null) {
            // 如果用户不存在，插入新的用户信息
            userInfoDao.saveUserInfo(userInfo)
        }
    }
}
