package com.kaixed.kchat.manager

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.kaixed.kchat.data.DataBase.getContactByUsername
import com.kaixed.kchat.data.event.UnreadEvent
import com.kaixed.kchat.data.local.AppDatabase
import com.kaixed.kchat.data.local.entity.Conversation
import com.kaixed.kchat.data.local.entity.Messages
import com.kaixed.kchat.utils.ConstantsUtils.getCurrentContactId
import com.kaixed.kchat.utils.ConstantsUtils.getUsername
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import org.greenrobot.eventbus.EventBus

/**
 * @Author: kaixed
 * @Date: 2024/12/13 10:42
 */
object ConversationManager {

    private val _conversationLiveData = MutableLiveData<List<Conversation>?>()
    val conversations: LiveData<List<Conversation>?> get() = _conversationLiveData

    private val conversationMap: MutableMap<String, Conversation> = mutableMapOf()

    private val conversationDao = AppDatabase.getDatabase().conversationDao()

    private var unreadCount = 0
    val unReadMsgCount get() = unreadCount

    init {
        loadConversations()
    }

    private fun loadConversations() {
        val scope = CoroutineScope(Dispatchers.IO)
        scope.launch {
            val conversations = conversationDao.getConversations()

            conversations.forEach { conversation ->
                if (conversation.unreadCount > 0) {
                    unreadCount += conversation.unreadCount
                }
                conversationMap[conversation.talkerId] = conversation
            }
            updateLiveData()
        }
    }

    suspend fun handleMessages(messages: Messages) {
        val talkerId = getCurrentContactId()
        if (messages.senderId != getUsername() && messages.talkerId != talkerId) {
            unreadCount++
        }
        val updatedConversation = updateConversation(messages)
        conversationDao.updateConversation(updatedConversation)
        updateLiveData()
    }

    fun hideConversation(contactId: String) {
        var count = 0
        conversationMap[contactId]?.apply {
            isShow = false
            count = unreadCount
        }
        if (count > 0) {
            unreadCount -= count
        }
        updateLiveData()
    }

    fun pinnedConversation(contactId: String, pinned: Boolean) {
        conversationMap[contactId]?.apply {
            isPinned = !pinned
        }
        updateLiveData()
    }

    suspend fun deleteConversation(conversation: Conversation) {
        val count = conversationMap[conversation.talkerId]?.unreadCount
        count?.let {
            if (it > 0) {
                unreadCount -= it
            }
        }
        conversationMap.remove(conversation.talkerId)
        conversationDao.deleteConversationsByContactId(conversation.talkerId)
        val messagesDao = AppDatabase.getDatabase().messagesDao()
        messagesDao.deleteMessagesByContactId(conversation.talkerId)
        updateLiveData()
    }

    suspend fun cleanUnreadCount(talkerId: String) {
        val con = conversationMap[talkerId]
        unreadCount -= con?.unreadCount ?: 0
        conversationMap[talkerId].apply {
            this?.unreadCount = 0
        }
        val conversation = conversationDao.getConversationByUsername(talkerId)

        conversation?.apply {
            unreadCount = 0
            conversationDao.updateConversation(this)
        }
        updateLiveData()
    }

    private val contactDao = AppDatabase.getDatabase().contactDao()

    suspend fun updateDisturbStatus(contactId: String, isDisturb: Boolean) {
        conversationMap[contactId]?.apply {
            doNotDisturb = isDisturb
            conversationDao.updateConversation(this)
        }

        val dbContact =
            contactDao.getContactByUsername(contactId).apply {
                this?.doNotDisturb = isDisturb
            }
        contactDao.updateContact(dbContact!!)
        updateLiveData()
    }

    suspend fun updateUnreadCount(talkerId: String, isRead: Boolean) {
        unreadCount = if (isRead) unreadCount - 1 else unreadCount + 1
        val conversation = conversationMap[talkerId]?.apply {
            this.unreadCount = if (isRead) 1 else 0
        }
        if (conversation != null) {
            conversationDao.insertConversation(conversation)
        }
        updateLiveData()
    }

    private fun updateLiveData() {
        val unreadEvent = UnreadEvent(unreadCount)
        EventBus.getDefault().post(unreadEvent)
        _conversationLiveData.postValue(conversationMap.values.sortedByDescending { it.timestamp })
    }

    private suspend fun updateConversation(messages: Messages): Conversation {
        val existingConversation = conversationMap[messages.talkerId]
        return existingConversation?.apply {
            lastContent = if (messages.type == "4") "[图片]" else messages.content
            timestamp = messages.timestamp
            unreadCount =
                if (talkerId == getCurrentContactId() || messages.senderId == getUsername()) 0 else unreadCount + 1
        } ?: createConversation(messages)
    }

    private suspend fun createConversation(messages: Messages): Conversation {
        val contact = getContactByUsername(messages.talkerId)
        return Conversation(
            talkerId = messages.talkerId,
            nickname = contact?.remark ?: messages.talkerId,
            lastContent = if (messages.type == "4") "[图片]" else messages.content,
            timestamp = messages.timestamp,
            avatarUrl = messages.avatarUrl,
            unreadCount = if (messages.senderId == getUsername()) 0 else 1
        ).also {
            conversationMap[messages.talkerId] = it
        }
    }
}
