package com.kaixed.kchat.manager

import android.content.Context
import android.net.ConnectivityManager
import android.net.Network
import android.net.NetworkCapabilities
import android.net.NetworkRequest
import com.kaixed.kchat.App

class NetworkStateManager(context: Context) {
    private val connectivityManager =
        context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager

    private val networkCallback = object : ConnectivityManager.NetworkCallback() {
        override fun onAvailable(network: Network) {
            currentNetworkState = NetworkState.CONNECTED
            notifyListeners()
        }

        override fun onLost(network: Network) {
            currentNetworkState = NetworkState.DISCONNECTED
            notifyListeners()
        }

        override fun onCapabilitiesChanged(network: Network, capabilities: NetworkCapabilities) {
            // 判断网络类型和质量
            currentNetworkState = when {
                capabilities.hasTransport(NetworkCapabilities.TRANSPORT_WIFI) -> NetworkState.WIFI
                capabilities.hasTransport(NetworkCapabilities.TRANSPORT_CELLULAR) -> NetworkState.CELLULAR
                else -> NetworkState.CONNECTED
            }

            // 判断信号强度
            signalStrength = when {
                capabilities.hasCapability(NetworkCapabilities.NET_CAPABILITY_NOT_CONGESTED)
                        && capabilities.linkDownstreamBandwidthKbps > 2000 -> SignalStrength.STRONG

                capabilities.linkDownstreamBandwidthKbps < 500 -> SignalStrength.WEAK
                else -> SignalStrength.NORMAL
            }

            notifyListeners()
        }
    }

    private var currentNetworkState: NetworkState = NetworkState.UNKNOWN
    private var signalStrength: SignalStrength = SignalStrength.UNKNOWN
    private val listeners = mutableListOf<NetworkStateListener>()

    fun startListening() {
        val request = NetworkRequest.Builder().build()
        connectivityManager.registerNetworkCallback(request, networkCallback)
    }

    fun stopListening() {
        connectivityManager.unregisterNetworkCallback(networkCallback)
    }

    fun getCurrentNetworkInfo(): Pair<NetworkState, SignalStrength> {
        return Pair(currentNetworkState, signalStrength)
    }

    fun addListener(listener: NetworkStateListener) {
        listeners.add(listener)
    }

    fun removeListener(listener: NetworkStateListener) {
        listeners.remove(listener)
    }

    private fun notifyListeners() {
        listeners.forEach { it.onNetworkStateChanged(currentNetworkState, signalStrength) }
    }

    enum class NetworkState {
        UNKNOWN, CONNECTED, DISCONNECTED, WIFI, CELLULAR
    }

    enum class SignalStrength {
        UNKNOWN, WEAK, NORMAL, STRONG
    }

    interface NetworkStateListener {
        fun onNetworkStateChanged(state: NetworkState, strength: SignalStrength)
    }
}