package com.kaixed.kchat.network.interceptor

import android.util.Base64
import android.util.Log
import com.kaixed.kchat.network.ApiCall
import com.kaixed.kchat.network.RetrofitClient
import com.kaixed.kchat.utils.CacheUtils
import com.kaixed.kchat.utils.ConstantsUtils.getUsername
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.runBlocking
import kotlinx.coroutines.withContext
import okhttp3.Interceptor
import okhttp3.Response
import org.json.JSONObject
import java.util.concurrent.locks.ReentrantLock
import kotlin.concurrent.withLock

class TokenRefreshInterceptor() : Interceptor {

    private val lock = ReentrantLock()

    private val authApiService by lazy { RetrofitClient.authApiService }

    override fun intercept(chain: Interceptor.Chain): Response {
        // 获取当前的 accessToken
        var accessToken = CacheUtils.getAccessToken()
        var refreshToken = CacheUtils.getRefreshToken()
        val expiration = parseTokenManually(accessToken)
        val refreshTokenExpiration = parseTokenManually(refreshToken)
        val now = System.currentTimeMillis() / 1000

        // 如果 Token 距离过期时间小于 5 分钟，刷新 Token
        if (expiration - now < 300) {
            lock.withLock {
                // Double-check 防止并发刷新
                val latestAccessToken = CacheUtils.getAccessToken()
                if (latestAccessToken == accessToken) {
                    // 调用刷新 Token 方法
                    runBlocking {
                        refreshAccessToken() // 刷新 Refresh Token
                    }
                } else {
                    accessToken = latestAccessToken
                }
            }
        }

        // 判断 Refresh Token 是否即将过期（提前一天刷新）
        if (refreshTokenExpiration - now < 24 * 60 * 60) { // 剩余小于24小时
            lock.withLock {
                val latestRefreshToken = CacheUtils.getRefreshToken()
                if (latestRefreshToken == refreshToken) {
                    runBlocking {
                        refreshRefreshToken() // 刷新 Refresh Token
                    }
                }
            }
        }

        // 将最新的 Token 添加到请求头
        val newRequest = chain.request().newBuilder()
            .addHeader("Authorization", "Bearer $accessToken")
            .build()

        return chain.proceed(newRequest)
    }

    private suspend fun refreshAccessToken() {
        return withContext(Dispatchers.IO) {
            try {
                ApiCall.apiCall(
                    apiCall = { authApiService.auth("Bearer ${CacheUtils.getRefreshToken()}", getUsername()) },
                    errorMessage = "刷新Token失败"
                ).onSuccess {
                    it?.let {
                        CacheUtils.setAccessToken(it)
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    private suspend fun refreshRefreshToken() {
        return withContext(Dispatchers.IO) {
            try {
                ApiCall.apiCall(
                    apiCall = {
                        authApiService.refresh(
                            CacheUtils.getRefreshToken(),
                            getUsername()
                        )
                    },
                    errorMessage = "刷新Token失败"
                ).onSuccess {
                    it?.let {
                        CacheUtils.setRefreshToken(it)
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    private fun parseTokenManually(token: String): Long {
        var expiration = 0L
        try {
            // JWT 的有效载荷部分是第二部分
            val payload = token.split(".")[1]

            // 解码Base64编码的字符串
            val decodedBytes = Base64.decode(payload, Base64.URL_SAFE or Base64.NO_WRAP)
            val decodedString = String(decodedBytes)

            // 将解码后的字符串转为JSONObject
            val jsonObject = JSONObject(decodedString)

            expiration = jsonObject.optLong("exp") // 获取过期时间
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return expiration
    }
}
