package com.kaixed.kchat.send

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.SupervisorJob
import kotlinx.coroutines.delay
import kotlinx.coroutines.isActive
import kotlinx.coroutines.launch
import kotlinx.coroutines.sync.Semaphore
import kotlinx.coroutines.withTimeoutOrNull
import kotlin.math.pow
import kotlin.random.Random

class UploadWorker(private val semaphore: Semaphore) {

    // 模拟消息上传接口
    suspend fun upload(task: MessageTask): Boolean {
        // 模拟上传耗时 & 网络请求，可用 Retrofit 或 OkHttp 实现
        delay(500L)  // 模拟网络延迟
        // 模拟成功率 80%
        return Random.nextInt(100) < 80
    }
}

class MessageUploadManager(
    private val retryQueue: RetryQueueManager,
    private val worker: UploadWorker,
    private val ackTimeout: Long = 5000L // ACK 超时设为 5 秒
) {
    private val scope = CoroutineScope(Dispatchers.IO + SupervisorJob())

    // 限制并发上传数量，例如最多 3 个同时上传
    private val semaphore = Semaphore(3)

    // 启动调度流程，监视重传队列
    fun start() {
        // 持续监听任务队列
        scope.launch {
            while (isActive) {
                // 检查网络状态（这里应嵌入 NetworkMonitor 判断，可扩展为挂起函数）
                if (!isNetworkAvailable()) {
                    delay(2000L)
                    continue
                }

                // 从队列中选择可上传任务
                val task = retryQueue.getReadyTask()
                if (task != null) {
                    // 进入发送状态
                    task.state = MessageState.SENDING
                    retryQueue.updateTask(task)

                    // 限流并发执行上传任务
                    semaphore.acquire()
                    launch {
                        val success = worker.upload(task)
                        // 模拟等待 ACK 的逻辑
                        // 如果上传成功，则等待 ackTimeout 时间后检查是否收到 ACK
                        if (success) {
                            // 调用 ackWaitTimer 监听 ACK
                            withTimeoutOrNull(ackTimeout) {
                                waitForAck(task.id.toString())
                            }?.let {
                                // 成功收到 ACK
                                task.state = MessageState.SUCCESS
                                retryQueue.updateTask(task)
                                retryQueue.removeTask(task)
                            } ?: run {
                                // 超时未收到 ACK，则作为失败处理
                                retryTask(task)
                            }
                        } else {
                            // 上传失败处理
                            retryTask(task)
                        }
                        semaphore.release()
                    }
                } else {
                    // 无任务时稍作延迟
                    delay(1000L)
                }
            }
        }
    }

    // 模拟网络状态检测函数，可结合 ConnectivityManager 使用
    private fun isNetworkAvailable(): Boolean {
        // 实际应检测网络连接
        return true
    }

    // 模拟等待 ACK，实际可通过 Channel 或 WebSocket 收到 ack 后返回
    private suspend fun waitForAck(taskId: String) {
        // 这里模拟 ack 延迟，实际情况中应由 AckHandler 回调触发
        delay(300L)  // 模拟短延时后收到 ACK
    }

    // 重试逻辑：更新重试次数、计算下次重试时间，然后回到队列
    private fun retryTask(task: MessageTask) {
        task.retryCount++
        if (task.retryCount > task.maxRetry) {
            task.state = MessageState.FAILED
            retryQueue.updateTask(task)
            // 可在此处上报失败或通知用户
        } else {
            // 计算指数退避时间（如基于 2000ms 基数，再加上少量随机抖动）
            val newDelay =
                (2000L * 2.0.pow(task.retryCount.toDouble())).toLong() + Random.nextLong(500)
            task.nextRetryTime = System.currentTimeMillis() + newDelay
            task.state = MessageState.PENDING
            retryQueue.updateTask(task)
        }
    }

    // 当收到 ACK 时调用，由 AckHandler 或网络消息回调触发
    fun onAckReceived(taskId: Long) {
        // 查找任务并更新状态
        val task = retryQueue.taskFlow.value.find { it.id == taskId }
        if (task != null) {
            task.state = MessageState.SUCCESS
            retryQueue.updateTask(task)
            retryQueue.removeTask(task)
        }
    }
}