package com.kaixed.kchat.service

import android.util.Log
import com.google.gson.Gson
import com.kaixed.kchat.data.local.entity.Messages
import com.kaixed.kchat.network.NetworkInterface.WEBSOCKET
import com.kaixed.kchat.network.NetworkInterface.WEBSOCKET_SERVER_URL
import com.kaixed.kchat.network.OkhttpHelper
import com.kaixed.kchat.utils.Constants.MMKV_COMMON_DATA
import com.kaixed.kchat.utils.ConstantsUtils
import com.kaixed.kchat.utils.ConstantsUtils.getUsername
import com.tencent.mmkv.MMKV
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.SupervisorJob
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import okhttp3.Request
import okhttp3.Response
import okhttp3.WebSocket
import okhttp3.WebSocketListener

object OkHttpWebSocketClient : WebSocketClient {

    private val scope: CoroutineScope = CoroutineScope(Dispatchers.IO + SupervisorJob())
    private val url: String = "$WEBSOCKET_SERVER_URL$WEBSOCKET${getUsername()}"
    private lateinit var webSocket: WebSocket
    private var messageListener: ((String) -> Unit)? = null
    private var isConnected = false

    private const val TAG = "WebSocketService"
    private const val WEBSOCKET_CLOSE_CODE = 1000

    override fun connect() {
        establishConnection()
    }

    override fun send(message: String, msgLocalId: Long): Boolean {
        return if (isConnected) {
            val kv = MMKV.mmkvWithID(MMKV_COMMON_DATA)
            kv.putLong("msgLocalId", msgLocalId)
            webSocket.send(message)
        } else {
            false
        }
    }

    override fun close(code: Int, reason: String) {
        webSocket.close(code, reason)
        isConnected = false
    }

    override fun setMessageListener(listener: (String) -> Unit) {
        this.messageListener = listener
    }

    override fun isConnected(): Boolean = isConnected

    private fun reconnect(attempt: Int = 1) {
        scope.launch {
            val delayMs = minOf(attempt * 1000L, 30000L) // 指数退避，最大 30 秒
            delay(delayMs)
            establishConnection()
            if (!isConnected()) {
                reconnect(attempt + 1)
            }
        }
    }

    private fun establishConnection() {
        val request = Request.Builder()
            .url(url).build()
        val listener = EchoWebSocketListener()
        val client = OkhttpHelper.getInstance()
        webSocket = client.newWebSocket(request, listener)
    }

    private class EchoWebSocketListener : WebSocketListener() {
        override fun onOpen(webSocket: WebSocket, response: Response) {
            Log.d(TAG, "WebSocket Opened")
            isConnected = true
        }

        override fun onMessage(webSocket: WebSocket, text: String) {
            messageListener?.invoke(text)
            val messages = Gson().fromJson(text, Messages::class.java)

//            serviceScope.launch {
//                if ("ack" == messages.type) {
//                    val existedMessage =
//                        messagesDao.getMessageByLocalId(messages.msgLocalId)
//
//                    existedMessage.apply {
//                        timestamp = messages.timestamp
//                        if (messages.msgSvrId == 0L) {
//                            messages.msgSvrId = messages.msgLocalId
//                        }
//                        messagesDao.updateMessage(this)
//                    }
//                } else {
//                    messages.talkerId = messages.senderId
//                    _messagesMutableLiveData.postValue(messages)
//                    MessageProcessor.processorMsg(messages)
//                    messagesDao.insert(messages)
//                    ConversationManager.handleMessages(messages)
//                }
//            }
        }

        override fun onClosing(webSocket: WebSocket, code: Int, reason: String) {
            isConnected = false
            webSocket.close(WEBSOCKET_CLOSE_CODE, null)
            Log.d(TAG, "WebSocket closing: $reason")
            establishConnection()
        }

        override fun onFailure(webSocket: WebSocket, t: Throwable, response: Response?) {
            isConnected = false
            Log.d(TAG, "WebSocket closing: ${t.cause}")
            Log.d(TAG, "WebSocket closing: $t")
            establishConnection()
        }
    }
}