package com.kaixed.kchat.ui.activity

import android.annotation.SuppressLint
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.graphics.Color
import android.os.Bundle
import android.os.Handler
import android.os.IBinder
import android.os.Looper
import android.util.Log
import android.view.KeyEvent
import android.view.MotionEvent
import android.view.View
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.LinearLayout
import androidx.activity.OnBackPressedCallback
import androidx.activity.enableEdgeToEdge
import androidx.activity.viewModels
import androidx.core.graphics.drawable.toDrawable
import androidx.core.widget.addTextChangedListener
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.drake.softinput.getSoftInputHeight
import com.drake.softinput.hasSoftInput
import com.kaixed.core.common.base.BaseActivity
import com.kaixed.kchat.R
import com.kaixed.kchat.data.local.AppDatabase
import com.kaixed.kchat.data.local.entity.Messages
import com.kaixed.kchat.data.model.FunctionItem
import com.kaixed.kchat.databinding.ActivityChatBinding
import com.kaixed.kchat.manager.ConversationManager
import com.kaixed.kchat.manager.MessagesManager
import com.kaixed.kchat.processor.MessageProcessor
import com.kaixed.kchat.service.ChatViewModel
import com.kaixed.kchat.service.WebSocketService
import com.kaixed.kchat.service.WebSocketService.LocalBinder
import com.kaixed.kchat.ui.adapter.ChatAdapter
import com.kaixed.kchat.ui.adapter.EmojiAdapter
import com.kaixed.kchat.ui.adapter.FunctionPanelAdapter
import com.kaixed.kchat.ui.i.IOnItemClickListener
import com.kaixed.kchat.ui.i.OnItemClickListener
import com.kaixed.kchat.ui.widget.LoadingDialogFragment
import com.kaixed.kchat.utils.Constants.CURRENT_CONTACT_ID
import com.kaixed.kchat.utils.Constants.KEYBOARD_HEIGHT
import com.kaixed.kchat.utils.Constants.MMKV_USER_SESSION
import com.kaixed.kchat.utils.ConstantsUtils.getKeyboardHeight
import com.kaixed.kchat.utils.ConstantsUtils.getUsername
import com.kaixed.kchat.utils.ImageEngines
import com.kaixed.kchat.utils.ImageSpanUtil.insertEmoji
import com.kaixed.kchat.viewmodel.FileViewModel
import com.luck.picture.lib.basic.PictureSelector
import com.luck.picture.lib.config.SelectMimeType
import com.luck.picture.lib.entity.LocalMedia
import com.luck.picture.lib.interfaces.OnResultCallbackListener
import com.tencent.mmkv.MMKV
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import org.json.JSONObject
import java.io.File
import kotlin.math.max

class ChatActivity : BaseActivity<ActivityChatBinding, ChatViewModel>(), OnItemClickListener,
    IOnItemClickListener {

    private var chatAdapter: ChatAdapter? = null

    private var webSocketService: WebSocketService? = null

    private val messagesDao = AppDatabase.getDatabase().messagesDao()

    private var contactId: String = ""

    private var contactNickname: String? = null

    private val username: String by lazy { getUsername() }

    private val context: Context = this

    private var strings: MutableList<String>? = mutableListOf()

    private var imm: InputMethodManager? = null

    private var softKeyboardHeight = 0

    private var bound = false

    private val mmkv by lazy { MMKV.mmkvWithID(MMKV_USER_SESSION) }

    private val fileViewModel: FileViewModel by viewModels()

    private var isSearchHistory = false

    private var msgLocalId = 0L

    companion object {
        private const val UNBLOCK_DELAY_TIME = 200L
    }

    override fun inflateBinding(): ActivityChatBinding {
        return ActivityChatBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<ChatViewModel> {
        return ChatViewModel::class.java
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        MessagesManager.setContactId(contactId)
        firstLoadMessage()
        bindWebSocketService()
        setPanelChange()
//        if (isSearchHistory) {
//            val size = MessagesManager.queryHistory(msgLocalId)
//            binding.rvChatList.smoothScrollToPosition(size - 1)
//        }
    }

    private fun firstLoadMessage() {
        lifecycleScope.launch {
            withContext(Dispatchers.IO) {
                MessagesManager.firstLoadMessages()
            }
        }
    }

    override fun onResume() {
        super.onResume()
//        if (!isSearchHistory) {
//            firstLoadMessage()
//        }
    }

    private val connection: ServiceConnection = object : ServiceConnection {
        override fun onServiceConnected(className: ComponentName, service: IBinder) {
            val binder = service as LocalBinder
            webSocketService = binder.getService()
            bound = true
        }

        override fun onServiceDisconnected(arg0: ComponentName) {
            bound = false
        }
    }

    private fun setBackPressListener() {
        onBackPressedDispatcher.addCallback(this, object : OnBackPressedCallback(true) {
            override fun handleOnBackPressed() {
                if (binding.clBottomPanel.isShown) {
                    hidePanel(false)
                } else {
                    isEnabled = false
                    onBackPressedDispatcher.onBackPressed()
                }
            }
        })
    }

    @SuppressLint("ClickableViewAccessibility")
    private fun setPanelChange() {
        binding.etInput.setOnTouchListener { _, event ->
            if (event.action == MotionEvent.ACTION_UP && binding.clBottomPanel.isShown) {
                lockContentViewHeight()
                hidePanel(true)
                unlockContentViewHeight()
            }
            false
        }

        binding.ivFunctionPanel.setOnClickListener {
            if (binding.clBottomPanel.isShown) {
                if (binding.rvEmoji.isShown) {
                    handlePanelSwitch(false)
                } else {
                    hideBottomPanel()
                }
            } else {
                switchPanel(false)
            }
        }

        binding.ivEmoji.setOnClickListener {
            if (binding.clBottomPanel.isShown) {
                if (binding.gvFunctionPanel.isShown) {
                    handlePanelSwitch(true)
                } else {
                    hideBottomPanel()
                }
            } else {
                switchPanel(true)
            }
        }
    }

    private fun switchPanel(isSwitchToEmoji: Boolean) {
        if (hasSoftInput()) {
            lockContentViewHeight()
            handlePanelSwitch(isSwitchToEmoji)
            unlockContentViewHeight()
        } else {
            handlePanelSwitch(isSwitchToEmoji)
        }
    }

    private fun hideBottomPanel() {
        lockContentViewHeight()
        hidePanel(true)
        unlockContentViewHeight()
    }

    private fun lockContentViewHeight() {
        val layoutParams = binding.rvChatList.layoutParams as LinearLayout.LayoutParams
        layoutParams.apply {
            height = binding.rvChatList.height
            weight = 0f
            binding.rvChatList.layoutParams = this
        }
    }

    private fun unlockContentViewHeight() {
        binding.rvChatList.postDelayed({
            val layoutParams = binding.rvChatList.layoutParams as LinearLayout.LayoutParams
            layoutParams.apply {
                weight = 1F
                binding.rvChatList.layoutParams = this
            }
        }, UNBLOCK_DELAY_TIME)
    }

    private fun setSoftKeyboardVisibility(isShowKeyboard: Boolean) {
        imm = getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager
        if (isShowKeyboard) {
            binding.etInput.requestFocus()
            imm!!.showSoftInput(binding.etInput, 0)
        } else {
            imm!!.hideSoftInputFromWindow(binding.etInput.windowToken, 0)
        }
    }

    private fun handlePanelSwitch(isEmojiPanel: Boolean) {
        setSoftKeyboardVisibility(false)
        binding.clBottomPanel.visibility = View.VISIBLE

        binding.rvEmoji.visibility = if (isEmojiPanel) View.VISIBLE else View.INVISIBLE
        binding.gvFunctionPanel.visibility = if (isEmojiPanel) View.INVISIBLE else View.VISIBLE
    }

    private fun hidePanel(showSoftKeyboard: Boolean) {
        if (binding.clBottomPanel.isShown) {
            binding.clBottomPanel.visibility = View.GONE
            if (showSoftKeyboard) {
                setSoftKeyboardVisibility(true)
            }
        }
    }

    private fun setupFunctionPanel() {
        strings?.add("[委屈]")
        val mEmojiAdapter = EmojiAdapter(strings)
        mEmojiAdapter.setOnItemClickListener(this)
        binding.rvEmoji.adapter = mEmojiAdapter

        val gridLayoutManager = GridLayoutManager(this, 7)
        binding.rvEmoji.layoutManager = gridLayoutManager

        val map: MutableMap<String, Int> = mutableMapOf()

        map["相册"] = R.drawable.ic_filled_picture
        map["拍摄"] = R.drawable.ic_filled_camera
        map["位置"] = R.drawable.ic_filled_location
        map["名片"] = R.drawable.ic_filled_personal
        map["转账"] = R.drawable.ic_filled_transfer
        map["文件"] = R.drawable.ic_folder_filled
        map["语音输入"] = R.drawable.ic_filled_voiceinput
        map["我的收藏"] = R.drawable.ic_filled_favorites

        val functionItems = mutableListOf<FunctionItem>()
        for (i in map.keys) {
            functionItems.add(FunctionItem(i, map[i]!!))
        }

        val functionPanelAdapter = FunctionPanelAdapter(functionItems, this)
        functionPanelAdapter.setOnItemClickListener(this)
        binding.gvFunctionPanel.adapter = functionPanelAdapter
    }

    override fun onItemClick(position: Int) {
        runOnUiThread {
            val editable = binding.etInput.text
            // 获取当前光标位置
            val index = binding.etInput.selectionStart
            // 获取将要插入的表情符号
            val emoji = strings!![position]
            // 使用 ImageSpanUtil 插入表情符号
            insertEmoji(
                context, editable, binding.etInput.textSize.toInt(), emoji, index
            )
            // 设置新的光标位置
            binding.etInput.setSelection(index + emoji.length)
        }
    }

    override fun initView() {
        binding.ctb.setUnReadCount(ConversationManager.unReadMsgCount)
        setupFunctionPanel()
        setRecycleView()
        contactNickname?.let {
            binding.ctb.setTitleName(contactNickname!!)
        }
        setPanel()
    }

    private fun setPanel() {
        val layoutParams = binding.clBottomPanel.layoutParams
        layoutParams.height = softKeyboardHeight
        binding.clBottomPanel.layoutParams = layoutParams
    }

    override fun initData() {
        contactId = intent.getStringExtra("contactId").toString()
        contactNickname = intent.getStringExtra("contactNickname")
        isSearchHistory = intent.getBooleanExtra("isSearchHistory", false) == true
        msgLocalId = intent.getLongExtra("msgLocalId", 0)

        binding.ctb.setTitleName(contactNickname!!)

        softKeyboardHeight = getKeyboardHeight()
        mmkv.putString(CURRENT_CONTACT_ID, contactId)
    }

    private fun setRecycleView() {
        val layoutManager = LinearLayoutManager(this)
        layoutManager.reverseLayout = true
        binding.rvChatList.layoutManager = layoutManager
        chatAdapter = ChatAdapter(this)
        binding.rvChatList.adapter = chatAdapter
        binding.rvChatList.itemAnimator = null
    }

    private fun loadMoreMessages() {
        lifecycleScope.launch {
            withContext(Dispatchers.IO) {
                MessagesManager.loadMoreMessages()
            }
        }
    }

    override fun observeData() {
//        webSocketService!!.messageLivedata.observe(this) {
//            it?.let {
//                handleMsg(it)
//            }
//        }
        lifecycleScope.launch {
            MessagesManager.messages.collect {
                chatAdapter?.submitList(it)
                binding.rvChatList.post {
                    binding.rvChatList.smoothScrollToPosition(0)
                }
            }
        }
    }

    override fun setupListeners() {
        binding.etInput.setOnClickListener {
            if (hasSoftInput()) {
                MMKV.defaultMMKV().encode(KEYBOARD_HEIGHT, max(getSoftInputHeight(), 300))
            }
        }

        setBackPressListener()

        binding.gvFunctionPanel.selector = Color.TRANSPARENT.toDrawable()

        binding.tvSend.setOnClickListener {
            val message = binding.etInput.text.toString().trim()
            sendMessage(contactId, message, "0")
        }

        binding.rvChatList.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                if (!recyclerView.canScrollVertically(-1)) {
                    loadMoreMessages()
                }
            }
        })

        binding.ctb.setOnTitleClickListener {
            startActivity(Intent(context, ContactsDetailActivity::class.java).apply {
                putExtra("contactId", contactId)
            })
        }

        binding.ctb.setOnSettingClickListener {
            startActivity(Intent(context, ChatDetailActivity::class.java).apply {
                putExtra("contactId", contactId)
            })
        }

        binding.etInput.addTextChangedListener(afterTextChanged = { _ ->
            val isInputEmpty = binding.etInput.text.toString().isEmpty()
            binding.ivFunctionPanel.visibility = if (!isInputEmpty) View.GONE else View.VISIBLE
            binding.tvSend.visibility = if (isInputEmpty) View.GONE else View.VISIBLE
        })

        binding.etInput.setOnEditorActionListener { _, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_DONE || (event?.keyCode == KeyEvent.KEYCODE_ENTER && event.action == KeyEvent.ACTION_DOWN)) {
                val message = binding.etInput.text.toString().trim()
                sendMessage(contactId, message, "0")
                return@setOnEditorActionListener true
            }
            false
        }
    }

    private fun sendMessage(talkerId: String, message: String, type: String) {
        val messages = Messages(
            content = message,
            timestamp = System.currentTimeMillis(),
            senderId = username,
            talkerId = talkerId,
            type = type,
        )
        lifecycleScope.launch {

            withContext(Dispatchers.IO) {
                val msg = MessageProcessor.processorMsg(messages)
                val id: Long = messagesDao.insert(msg)

                val jsonObject = JSONObject()
                val jsonObject2 = JSONObject()
                jsonObject2.put("receiverId", contactId)
                jsonObject2.put("content", message)
                jsonObject2.put("msgLocalId", id)
                jsonObject.put("type", if (type == "4") "image" else "text")
                jsonObject.put("body", jsonObject2)

                viewModel.sendChatMessage(jsonObject.toString(), id)
//                webSocketService!!.storeOwnerMsg(messages)
                ConversationManager.handleMessages(messages)

                MessagesManager.sendMessages(messages)
            }

            binding.etInput.setText("")
            binding.rvChatList.post {
                binding.rvChatList.smoothScrollToPosition(0)
            }
        }

    }

    private fun bindWebSocketService() {
        bindService(
            Intent(
                context, WebSocketService::class.java
            ), connection, BIND_AUTO_CREATE
        )
    }

    override fun onDestroy() {
        super.onDestroy()
        MessagesManager.resetMessages()
        mmkv.putString(CURRENT_CONTACT_ID, "")
        if (bound) {
            unbindService(connection)
            bound = false
        }
    }

    private fun uploadFile(imagePath: String) {
        val loadingDialog: LoadingDialogFragment = LoadingDialogFragment.newInstance("正在上传...")

        loadingDialog.showLoading(supportFragmentManager)
        if (imagePath.isEmpty()) {
            return
        }
        val file = File(imagePath)
        fileViewModel.uploadFileResult.observe(this) { result ->
            result.onSuccess {
                sendMessage(contactId, it!!, "4")
                toast("上传成功")

            }

            result.onFailure {
                toast("上传失败")
            }

            loadingDialog.dismissLoading()
        }
        fileViewModel.uploadFile(file)
    }

    //TODO: 待优化成微信选择上传模式，当前上传模式耗费用户等待时间
    private fun selectPicture() {
        PictureSelector.create(this).openGallery(SelectMimeType.ofImage())
            .setImageEngine(ImageEngines())
            .forResult(object : OnResultCallbackListener<LocalMedia?> {
                override fun onResult(result: ArrayList<LocalMedia?>) {
                    result.forEach { localMedia ->
                        localMedia?.let {
                            val imagePath = it.realPath
                            val handler = Handler(Looper.getMainLooper())
                            handler.postDelayed({
                                uploadFile(imagePath)
                            }, 200L)
                            Log.d("haha", "图片路径: $imagePath")
                        }
                    }
                }

                override fun onCancel() {
                    toast("已取消")
                }
            })
    }

    override fun onFunctionItemClick(position: Int) {
        when (position) {
            0 -> {
                selectPicture()
            }
        }
    }
}
