package com.kaixed.kchat.ui.activity

import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import androidx.activity.enableEdgeToEdge
import androidx.activity.viewModels
import androidx.core.content.ContextCompat
import com.drake.softinput.getSoftInputHeight
import com.drake.softinput.hasSoftInput
import com.kaixed.kchat.R
import com.kaixed.kchat.data.DataBase
import com.kaixed.kchat.data.local.entity.UserInfo
import com.kaixed.kchat.databinding.ActivityLoginBinding
import com.kaixed.kchat.ui.base.BaseActivity
import com.kaixed.kchat.ui.widget.MyBottomSheetFragment
import com.kaixed.kchat.utils.Constants.ACCESS_TOKEN
import com.kaixed.kchat.utils.Constants.KEYBOARD_HEIGHT
import com.kaixed.kchat.utils.Constants.REFRESH_TOKEN
import com.kaixed.kchat.utils.DrawableUtil.createDrawable
import com.kaixed.kchat.utils.ScreenUtils.dp2px
import com.kaixed.kchat.viewmodel.UserViewModel
import com.tencent.mmkv.MMKV
import kotlin.math.max

class LoginActivity : BaseActivity<ActivityLoginBinding>() {

    private val userViewModel: UserViewModel by viewModels()

    private var isLoginByTelephone = true

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        initView()
        setListeners()
        setObservers()

        binding.tvLogin.setOnClickListener {
            val username =
                if (isLoginByTelephone) binding.etTelephone.text.toString() else binding.etUsername.text.toString()
            val password =
                if (isLoginByTelephone) binding.etTelephonePassword.text.toString() else binding.etPassword.text.toString()
            login(username, password)
        }
    }

    private fun setObservers() {
        userViewModel.loginResult.observe(this) { loginResult ->
            loginResult.onSuccess {
                it?.let { saveUserInfo(it) }
                navigateToMain()
            }
            loginResult.onFailure { toast(it.message.toString()) }
        }
    }

    override fun initView() {
        setupLoginView()
    }

    private fun setListeners() {
        binding.ivClose.setOnClickListener { finish() }
        binding.tvChangeLoginWay.setOnClickListener {
            isLoginByTelephone = !isLoginByTelephone
            setupLoginView()
            if (isLoginByTelephone) {
                binding.etTelephone.requestFocus()
            } else {
                binding.etUsername.requestFocus()
            }
        }
        binding.tvMore.setOnClickListener { showBottomSheet() }
    }

    private fun setupLoginView() {
        resetFields()
        updateViewBasedOnLoginType()
    }

    private fun resetFields() {
        if (isLoginByTelephone) {
            binding.etTelephone.setText("")
            binding.etTelephonePassword.setText("")
        } else {
            binding.etUsername.setText("")
            binding.etPassword.setText("")
        }
    }

    private fun updateViewBasedOnLoginType() {
        if (isLoginByTelephone) {
            configureForPhoneLogin()
        } else {
            configureForUsernameLogin()
        }
    }

    private fun configureForPhoneLogin() {
        binding.etTelephone.setOnClickListener {
            if (hasSoftInput()) {
                MMKV.defaultMMKV().encode(KEYBOARD_HEIGHT, max(getSoftInputHeight(), 400))
            }
        }
        binding.etTelephonePassword.addTextChangedListener(textWatcher)
        binding.etTelephone.addTextChangedListener(textWatcher)
        binding.clUsername.visibility = View.GONE
        binding.clTelephone.visibility = View.VISIBLE
        binding.tvTitle.text = "手机号登录"
        binding.tvUsername.text = "手机号"
        binding.tvChangeLoginWay.text = "用户名登录"
        binding.tvTip.text = "上述手机号仅用于登陆验证"
    }

    private fun configureForUsernameLogin() {
        binding.etUsername.setOnClickListener {
            if (hasSoftInput()) {
                MMKV.defaultMMKV().encode(KEYBOARD_HEIGHT, max(getSoftInputHeight(), 400))
            }
        }
        binding.etPassword.addTextChangedListener(textWatcher)
        binding.etUsername.addTextChangedListener(textWatcher)
        binding.clUsername.visibility = View.VISIBLE
        binding.clTelephone.visibility = View.GONE
        binding.tvTitle.text = "用户名登录"
        binding.tvUsername.text = "用户名"
        binding.tvChangeLoginWay.text = "手机号登录"
        binding.tvTip.text = "上述账号仅用于登陆验证"
    }

    private fun login(username: String, password: String) {
        if (isInputValid(username, password)) {
            if (isLoginByTelephone) {
                userViewModel.loginByTelephone(username, password)
            } else {
                userViewModel.loginByUsername(username, password)
            }
        }
    }

    private fun isInputValid(username: String, password: String): Boolean {
        if (username.isEmpty() || password.isEmpty()) {
            toast("请输入${if (!isLoginByTelephone) "用户名" else "手机号"}或密码")
            return false
        }
        if (isLoginByTelephone && username.length != 11) {
            toast("请输入正确的手机号码")
            return false
        }
        return true
    }

    //TODO 存储用户信息
    private fun saveUserInfo(user: UserInfo) {
        DataBase.saveUserInfo(user)
        MMKV.defaultMMKV().putString(ACCESS_TOKEN, user.accessToken)
        MMKV.defaultMMKV().putString(REFRESH_TOKEN, user.refreshToken)
    }

    private fun showBottomSheet() {
        val bottomSheetFragment = MyBottomSheetFragment().apply {
            arguments = Bundle().apply {
                putStringArrayList("list", arrayListOf("紧急冻结", "安全中心", "反馈问题", "取消"))
            }
        }
        bottomSheetFragment.show(supportFragmentManager, bottomSheetFragment.tag)
    }

    private val textWatcher = object : TextWatcher {
        override fun beforeTextChanged(s: CharSequence, start: Int, count: Int, after: Int) {}
        override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}
        override fun afterTextChanged(s: Editable) {
            val isInputValid = when {
                isLoginByTelephone -> binding.etTelephonePassword.text.isNotEmpty() && binding.etTelephone.text.length == 11
                else -> binding.etUsername.text.isNotEmpty() && binding.etPassword.text.isNotEmpty()
            }
            updateLoginButtonState(isInputValid)
        }
    }

    private fun updateLoginButtonState(isInputValid: Boolean) {
        binding.tvLogin.apply {
            isEnabled = isInputValid
            setTextColor(
                if (isInputValid) ContextCompat.getColor(
                    this@LoginActivity,
                    R.color.white
                ) else Color.parseColor("#B4B4B4")
            )
            background = createDrawable(
                if (isInputValid) ContextCompat.getColor(
                    this@LoginActivity,
                    R.color.green
                ) else Color.parseColor("#E1E1E1"), dp2px(8)
            )
        }
    }

    private fun navigateToMain() {
        toast("登录成功")
        startActivity(Intent(this, MainActivity::class.java))
        finish()
    }

    override fun onDestroy() {
        super.onDestroy()
        binding.root.viewTreeObserver.removeOnGlobalLayoutListener { }
    }

    override fun initData() {}
    override fun observeData() {
    }

    override fun setupListeners() {
    }

    override fun inflateBinding(): ActivityLoginBinding {
        return ActivityLoginBinding.inflate(layoutInflater)
    }
}
