package com.kaixed.kchat.ui.activity

import android.Manifest
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import androidx.activity.enableEdgeToEdge
import androidx.activity.viewModels
import androidx.core.content.ContextCompat
import androidx.core.view.WindowCompat
import androidx.fragment.app.Fragment
import androidx.lifecycle.lifecycleScope
import androidx.viewpager2.adapter.FragmentStateAdapter
import androidx.viewpager2.widget.ViewPager2
import com.huawei.hms.hmsscankit.ScanUtil
import com.huawei.hms.ml.scan.HmsScan
import com.kaixed.kchat.R
import com.kaixed.kchat.data.DataBase
import com.kaixed.kchat.data.event.UnreadEvent
import com.kaixed.kchat.data.local.AppDatabase
import com.kaixed.kchat.data.local.dao.MessagesDao
import com.kaixed.kchat.databinding.ActivityMainBinding
import com.kaixed.kchat.ui.base.BaseActivity
import com.kaixed.kchat.ui.fragment.ContactFragment
import com.kaixed.kchat.ui.fragment.DiscoveryFragment
import com.kaixed.kchat.ui.fragment.HomeFragment
import com.kaixed.kchat.ui.fragment.MineFragment
import com.kaixed.kchat.ui.widget.LoadingDialogFragment
import com.kaixed.kchat.utils.ConstantsUtils.getUsername
import com.kaixed.kchat.utils.ConstantsUtils.isFirstLaunchApp
import com.kaixed.kchat.viewmodel.ContactViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode


class MainActivity : BaseActivity<ActivityMainBinding>() {

    private val colorPrimary: Int by lazy { ContextCompat.getColor(this, R.color.green) }

    private val colorBlack: Int by lazy { ContextCompat.getColor(this, R.color.black) }

    private val contactViewModel: ContactViewModel by viewModels()

    private val navBinding by lazy { binding.bottomNavContainer }

    private val loadingDialogFragment by lazy { LoadingDialogFragment.newInstance("同步数据中") }

    private val messagesDao: MessagesDao = AppDatabase.getDatabase().messagesDao()

    private val navItems by lazy {
        listOf(
            NavItem(navBinding.ivHome, navBinding.tvHome),
            NavItem(navBinding.ivContact, navBinding.tvContact),
            NavItem(navBinding.ivDiscovery, navBinding.tvDiscovery),
            NavItem(navBinding.ivMine, navBinding.tvMine)
        )
    }

    private val fragments = listOf<Fragment>(
        HomeFragment(),
        ContactFragment(),
        DiscoveryFragment(),
        MineFragment()
    )

    private lateinit var viewPagerAdapter: FragmentStateAdapter

    override fun inflateBinding(): ActivityMainBinding {
        return ActivityMainBinding.inflate(layoutInflater)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        WindowCompat.setDecorFitsSystemWindows(window, false)
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()

        EventBus.getDefault().register(this)

        initView()

        initViewPager()
        setListener()
    }

    // 待优化成自定义界面，目前使用的华为sdk的默认界面
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode != RESULT_OK || data == null) {
            return
        }

        val errorCode =
            data.getIntExtra(ScanUtil.RESULT_CODE, ScanUtil.SUCCESS)

        when(errorCode) {
            ScanUtil.SUCCESS -> {
                val hmsScan: HmsScan? = data.getParcelableExtra(ScanUtil.RESULT)
                hmsScan?.let {
                    lifecycleScope.launch {
                        handleResult(it.showResult)
                    }
                }
            }
            ScanUtil.ERROR_NO_READ_PERMISSION -> {
                ContextCompat.checkSelfPermission(
                    this,
                    Manifest.permission.READ_EXTERNAL_STORAGE
                )
            }
            else -> {
                Toast.makeText(this, "扫描失败", Toast.LENGTH_SHORT).show()
            }
        }
    }

    // 处理扫描结果
    private suspend fun handleResult(result: String) {
        if (result.startsWith("kchat@")) {
            val contactId = result.substringAfter("kchat@")
            if (contactId == getUsername()) {
                startActivity(Intent(this, ContactsDetailActivity::class.java).apply {
                    putExtra("contactId", contactId)
                    putExtra("isMine", true)
                })
            }
            if (DataBase.isMyFriend(contactId)) {
                startActivity(Intent(this, ContactsDetailActivity::class.java).apply {
                    putExtra("contactId", contactId)
                })
            } else {
                contactViewModel.searchContact(contactId)
            }

        } else {
            Toast.makeText(this, "暂不支持识别其他内容", Toast.LENGTH_SHORT).show()
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        EventBus.getDefault().unregister(this)
    }

    private fun setListener() {
        contactViewModel.contactListResult.observe(this@MainActivity) { result ->
            result.onSuccess {
                //TODO 优化，使用协程
                val contactDao = AppDatabase.getDatabase().contactDao()
                lifecycleScope.launch {
                    withContext(Dispatchers.IO) {
                        contactDao.insertContacts(it ?: emptyList())
                        loadingDialogFragment.dismissLoading()
                    }
                }
            }
        }
        contactViewModel.searchContactResult.observe(this@MainActivity) { result ->
            result.onSuccess {
                val intent = Intent(this, ApplyFriendsDetailActivity::class.java).apply {
                    putExtra("user", it)
                }
                startActivity(intent)
            }

            result.onFailure {
                toast("暂不支持识别其他内容")
            }
        }
    }

    override fun initData() {
        if (isFirstLaunchApp()) {
            lifecycleScope.launch(Dispatchers.Main) {
                delay(200)
                loadingDialogFragment.showLoading(supportFragmentManager)
                contactViewModel.loadFriendList(getUsername())
            }
        }
    }

    override fun observeData() {
    }

    override fun setupListeners() {
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMessageEvent(unreadEvent: UnreadEvent) {
        val unreadCount = unreadEvent.unreadCount
        binding.bottomNavContainer.rlUnreadCount.visibility =
            if (unreadCount > 0) View.VISIBLE else View.INVISIBLE
        if (unreadCount > 0) {
            binding.bottomNavContainer.tvUnreadCount.text = unreadCount.toString()
        }
    }


    override fun initView() {
        navItems.forEachIndexed { index, navItem ->
            navItem.container.setOnClickListener {
                updateSelection(index)
            }
        }
    }

    fun updatePosition(position: Int) {
        updateSelection(position)
    }

    private fun initViewPager() {
        viewPagerAdapter = object : FragmentStateAdapter(this) {
            override fun getItemCount(): Int {
                return fragments.size
            }

            override fun createFragment(position: Int): Fragment {
                return fragments[position]
            }
        }
        binding.viewPager.adapter = viewPagerAdapter

        binding.viewPager.registerOnPageChangeCallback(object : ViewPager2.OnPageChangeCallback() {
            override fun onPageSelected(position: Int) {
                super.onPageSelected(position)
                updateSelection(position)
            }
        })
    }

    private fun updateSelection(position: Int) {
        binding.viewPager.currentItem = position
        navItems.forEachIndexed { index, navItem ->
            navItem.imageView.isSelected = index == position
            navItem.textView.setTextColor(if (index == position) colorPrimary else colorBlack)
        }
    }

    data class NavItem(
        val imageView: ImageView,
        val textView: TextView,
        val container: View = imageView.parent as View
    )
}
