package com.kaixed.kchat.ui.adapter

import android.content.Context
import android.util.Log
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.core.view.updateLayoutParams
import androidx.paging.PagingDataAdapter
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.drake.spannable.replaceSpan
import com.drake.spannable.span.CenterImageSpan
import com.kaixed.core.common.utils.TextUtil.extractDimensionsAndPrefix
import com.kaixed.kchat.R
import com.kaixed.kchat.data.local.entity.Messages
import com.kaixed.kchat.databinding.ChatRecycleItemCustomNormalBinding
import com.kaixed.kchat.databinding.ChatRecycleItemImageNormalBinding
import com.kaixed.kchat.databinding.ChatRecycleItemTipBinding
import com.kaixed.kchat.manager.MessagesManager
import com.kaixed.kchat.utils.ConstantsUtils
import com.kaixed.kchat.utils.ConstantsUtils.getUsername
import com.kaixed.kchat.utils.PopWindowUtil.showPopupWindow
import com.kaixed.kchat.utils.ViewUtil.changeTimerVisibility
import com.kaixed.kchat.utils.ViewUtil.setViewVisibility


class ChatAdapter(
    private val context: Context
) : ListAdapter<Messages, RecyclerView.ViewHolder>(DIFF_CALLBACK) {

    companion object {
        const val CUSTOM = 0
        const val TIP = 2
        const val IMAGE = 4
        const val VOICE = 6
        const val LOCATION = 8
        const val EMOJI = 10
        const val RED_PACKET = 12
        private const val TAG = "ChatAdapter"

        // 移动到外部作为静态对象
        private val DIFF_CALLBACK = object : DiffUtil.ItemCallback<Messages>() {
            override fun areItemsTheSame(oldItem: Messages, newItem: Messages): Boolean {
                val isSame = oldItem.msgLocalId == newItem.msgLocalId
                if (!isSame) {
                    Log.d(TAG, "Item changed: old=${oldItem.msgLocalId}, new=${newItem.msgLocalId}")
                }
                return isSame
            }

            override fun areContentsTheSame(oldItem: Messages, newItem: Messages): Boolean {
                val changed = oldItem != newItem
                if (changed) {
                    Log.d(TAG, "Item content changed: id=${oldItem.msgLocalId}")
                }
                return !changed
            }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        return when (viewType) {
            CUSTOM -> {
                CustomViewHolder(
                    ChatRecycleItemCustomNormalBinding.inflate(
                        LayoutInflater.from(parent.context), parent, false
                    )
                )
            }

            IMAGE -> {
                ImageViewHolder(
                    ChatRecycleItemImageNormalBinding.inflate(
                        LayoutInflater.from(parent.context), parent, false
                    )
                )
            }

            else -> {
                TipViewHolder(
                    ChatRecycleItemTipBinding.inflate(
                        LayoutInflater.from(parent.context), parent, false
                    )
                )
            }
        }
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        val singleMessage = getItem(position) ?: return
        when (holder) {
            is CustomViewHolder -> {
                holder.bindData(singleMessage)
            }

            is ImageViewHolder -> {
                holder.bindData(singleMessage)
            }

            is TipViewHolder -> {
                holder.bindData(singleMessage)
            }
        }
        (holder as? IHasTimer)?.let {
            changeTimerVisibility(it.getTimerView(), singleMessage)
        }
        handleLongClick(holder, position, singleMessage)
    }

    private fun handleLongClick(holder: RecyclerView.ViewHolder, position: Int, message: Messages) {
        val isMine = message.senderId == getUsername()
        val contentView = when (holder) {
            is CustomViewHolder -> if (isMine) holder.binding.tvMsgContentMine else holder.binding.tvMsgContent
            is ImageViewHolder -> if (isMine) holder.binding.imageMine else holder.binding.image
            else -> null
        }

        contentView?.setOnLongClickListener {
            val popupWindow = showPopupWindow(context, it, message.senderId == getUsername())
            val deleteButton = popupWindow.contentView.findViewById<TextView>(R.id.tv_delete)
            deleteButton.setOnClickListener {
                deleteMessage(message)
                popupWindow.dismiss()
            }
            true
        }
    }

    private fun deleteMessage(message: Messages) {
        MessagesManager.deleteMessage(message.msgLocalId)
    }

    interface IHasTimer {
        fun getTimerView(): TextView
    }

    class TipViewHolder(val binding: ChatRecycleItemTipBinding) :
        RecyclerView.ViewHolder(binding.root) {
        fun bindData(message: Messages) {
            if (message.show) {
                binding.tvTip.text = binding.root.context.getString(
                    R.string.withdraw_message_format, message.senderId
                )
            }
        }
    }

    class CustomViewHolder(val binding: ChatRecycleItemCustomNormalBinding) :
        RecyclerView.ViewHolder(binding.root), IHasTimer {
        fun bindData(message: Messages) {
            val sender = message.senderId == getUsername()

            setViewVisibility(
                parentView = binding.root,
                sender = sender,
                avatarId = binding.ifvAvatar.id,
                avatarMineId = binding.ifvAvatarMine.id,
                contentId = binding.tvMsgContent.id,
                contentMineId = binding.tvMsgContentMine.id
            )
            val avatarUrl = if (sender) ConstantsUtils.getAvatarUrl() else message.avatarUrl
            Glide.with(binding.root.context).load(avatarUrl)
                .into(if (sender) binding.ifvAvatarMine else binding.ifvAvatar)

            val contentView = if (sender) binding.tvMsgContentMine else binding.tvMsgContent

            contentView.text = message.content.replaceSpan("[委屈]") {
                CenterImageSpan(binding.root.context, R.drawable.emoji).setDrawableSize(55)
            }

        }

        override fun getTimerView(): TextView = binding.tvTimer
    }

    class ImageViewHolder(val binding: ChatRecycleItemImageNormalBinding) :
        RecyclerView.ViewHolder(binding.root), IHasTimer {
        fun bindData(message: Messages) {
            val sender = message.senderId == getUsername()
            val avatarUrl = if (sender) ConstantsUtils.getAvatarUrl() else message.avatarUrl
            Glide.with(binding.root.context).load(avatarUrl)
                .into(if (sender) binding.ifvAvatarMine else binding.ifvAvatar)

            setViewVisibility(
                parentView = binding.root,
                sender = sender,
                avatarId = binding.ifvAvatar.id,
                avatarMineId = binding.ifvAvatarMine.id,
                contentId = binding.image.id,
                contentMineId = binding.imageMine.id
            )
            val imageView = if (sender) binding.imageMine else binding.image
            loadImage(message, imageView)
        }

        private fun loadImage(message: Messages, imageView: ImageView) {
            val result = extractDimensionsAndPrefix(message.content)
            result?.let {
                val (url, width, height) = it
                imageView.apply {
                    updateLayoutParams {
                        this.height = height
                        this.width = width
                    }
                }
                Glide.with(binding.root.context).load(url).placeholder(R.drawable.image_loading)
                    .into(imageView)
            } ?: run {
                Glide.with(binding.root.context).load(message.content)
                    .placeholder(R.drawable.image_loading).into(imageView)
            }
        }

        override fun getTimerView(): TextView = binding.tvTimer
    }

    override fun getItemViewType(position: Int): Int {
        return getItem(position)?.type?.toInt() ?: 0
    }
}
