package com.kaixed.kchat.ui.widget

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.os.Build
import android.util.AttributeSet
import androidx.annotation.RequiresApi
import androidx.appcompat.widget.AppCompatEditText
import androidx.core.content.ContextCompat
import com.kaixed.kchat.R
import com.kaixed.kchat.utils.ScreenUtils

/**
 * 自定义 EditText 控件，用于绘制下划线。
 */
@RequiresApi(Build.VERSION_CODES.Q)
class CustomEditText @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    defStyle: Int = 0
) : AppCompatEditText(context, attrs, defStyle) {

    // 焦点时的下划线颜色
    private val focusedLineColor: Int by lazy { ContextCompat.getColor(context, R.color.green) }

    // 无焦点时的下划线颜色
    private val normalLineColor: Int by lazy { Color.parseColor("#E0E0E0") }

    // 下划线高度
    private val lineHeight: Float by lazy { ScreenUtils.dp2px(1).toFloat() }

    // 画笔，用于绘制下划线
    private var paint: Paint = Paint().apply {
        style = Paint.Style.STROKE
        strokeWidth = lineHeight
        color = normalLineColor // 初始颜色为无焦点时的颜色
    }

    init {
        // 确保背景设置为空以自定义下划线
        background = null

        // 设置焦点状态的参数，确保可以获得焦点
        isFocusable = true
        isFocusableInTouchMode = true

        textCursorDrawable = ContextCompat.getDrawable(context, R.drawable.cursor)
    }

    /**
     * 重写 onDraw() 方法绘制底部下划线
     */
    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)

        // 绘制底部的下划线
        val startX = 0f
        val startY = height.toFloat() // 在底部绘制下划线，距底部5dp
        val endX = width.toFloat()
        val endY = height.toFloat()

        canvas.drawLine(startX, startY, endX, endY, paint)
    }

    /**
     * 焦点变化时更新下划线的颜色
     */
    override fun onFocusChanged(
        focused: Boolean,
        direction: Int,
        previouslyFocusedRect: android.graphics.Rect?
    ) {
        super.onFocusChanged(focused, direction, previouslyFocusedRect)

        // 根据焦点状态更新下划线颜色
        paint.color = if (focused) {
            focusedLineColor
        } else {
            normalLineColor
        }

        // 强制重绘
        invalidate()
    }
}
