package com.kaixed.kchat.ui.widget

import android.content.Context
import android.graphics.Rect
import android.util.AttributeSet
import android.view.MotionEvent
import android.view.View
import android.view.animation.Animation
import android.view.animation.TranslateAnimation
import android.widget.ScrollView

class ReBoundScrollView @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : ScrollView(context, attrs, defStyleAttr) {

    private var mEnableTopRebound = true
    private var mEnableBottomRebound = true
    private var mOnReboundEndListener: OnReboundEndListener? = null
    private var mContentView: View? = null
    private val mRect = Rect()

    init {
        overScrollMode = View.OVER_SCROLL_NEVER
    }

    override fun onFinishInflate() {
        super.onFinishInflate()
        mContentView = getChildAt(0)
    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {
        super.onLayout(changed, l, t, r, b)
        mContentView?.let {
            mRect.set(it.left, it.top, it.right, it.bottom)
        }
    }

    fun setOnReboundEndListener(onReboundEndListener: OnReboundEndListener): ReBoundScrollView {
        this.mOnReboundEndListener = onReboundEndListener
        return this
    }

    fun setEnableTopRebound(enableTopRebound: Boolean): ReBoundScrollView {
        this.mEnableTopRebound = enableTopRebound
        return this
    }

    fun setEnableBottomRebound(enableBottomRebound: Boolean): ReBoundScrollView {
        this.mEnableBottomRebound = enableBottomRebound
        return this
    }

    private var lastY = 0
    private var rebound = false
    private var reboundDirection = 0 // <0: bottom rebound, >0: top rebound, 0: no rebound

    override fun dispatchTouchEvent(ev: MotionEvent): Boolean {
        mContentView?.let {
            when (ev.action) {
                MotionEvent.ACTION_DOWN -> {
                    lastY = ev.y.toInt()
                }

                MotionEvent.ACTION_MOVE -> {
                    if (!isScrollToTop() && !isScrollToBottom()) {
                        lastY = ev.y.toInt()
                        return super.dispatchTouchEvent(ev)
                    }
                    val deltaY = (ev.y - lastY).toInt()

                    if ((!mEnableTopRebound && deltaY > 0) || (!mEnableBottomRebound && deltaY < 0)) {
                        return super.dispatchTouchEvent(ev)
                    }

                    val offset = (deltaY * 0.48).toInt()
                    it.layout(mRect.left, mRect.top + offset, mRect.right, mRect.bottom + offset)
                    rebound = true
                }

                MotionEvent.ACTION_UP -> {
                    if (!rebound) return super.dispatchTouchEvent(ev)
                    reboundDirection = it.top - mRect.top
                    val animation =
                        TranslateAnimation(0f, 0f, it.top.toFloat(), mRect.top.toFloat())
                    animation.duration = 300
                    animation.setAnimationListener(object : Animation.AnimationListener {
                        override fun onAnimationStart(animation: Animation?) {}

                        override fun onAnimationEnd(animation: Animation?) {
                            mOnReboundEndListener?.let {
                                when {
                                    reboundDirection > 0 -> it.onReboundTopComplete()
                                    reboundDirection < 0 -> it.onReboundBottomComplete()
                                }
                            }
                            reboundDirection = 0
                        }

                        override fun onAnimationRepeat(animation: Animation?) {}
                    })
                    it.startAnimation(animation)
                    it.layout(mRect.left, mRect.top, mRect.right, mRect.bottom)
                    rebound = false
                }
            }
        }
        return super.dispatchTouchEvent(ev)
    }

    override fun setFillViewport(fillViewport: Boolean) {
        super.setFillViewport(true) // 默认填充 ScrollView 或者在 XML 布局中设置 fillViewport 属性
    }

    private fun isScrollToTop(): Boolean {
        return scrollY == 0
    }

    private fun isScrollToBottom(): Boolean {
        mContentView?.let {
            return it.height <= height + scrollY
        }
        return false
    }

    interface OnReboundEndListener {
        fun onReboundTopComplete()
        fun onReboundBottomComplete()
    }
}
