package com.kaixed.kchat.ui.widget

import android.animation.ValueAnimator
import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.Path
import android.graphics.Point
import android.os.Bundle
import android.os.Parcelable
import android.util.AttributeSet
import android.view.View
import android.view.animation.LinearInterpolator
import android.widget.Checkable
import com.kaixed.kchat.R

/**
 * Author : andy
 * Date   : 16/1/21 11:28
 * Email  : andyxialm@gmail.com
 * Github : github.com/andyxialm
 * Description : A custom CheckBox with animation for Android
 */
class SmoothCheckBox @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : View(context, attrs, defStyleAttr), Checkable {

    private val KEY_INSTANCE_STATE = "InstanceState"

    private val COLOR_TICK = Color.WHITE
    private val COLOR_UNCHECKED = Color.WHITE
    private val COLOR_CHECKED = Color.parseColor("#07C160")
    private val COLOR_FLOOR_UNCHECKED = Color.parseColor("#DFDFDF")

    private val DEF_DRAW_SIZE = 25
    private val DEF_ANIM_DURATION = 300

    private lateinit var mPaint: Paint
    private lateinit var mTickPaint: Paint
    private lateinit var mFloorPaint: Paint
    private lateinit var mTickPath: Path

    private lateinit var mTickPoints: Array<Point>
    private lateinit var mCenterPoint: Point

    private var mLeftLineDistance = 0f
    private var mRightLineDistance = 0f
    private var mDrewDistance = 0f
    private var mScaleVal = 1.0f
    private var mFloorScale = 1.0f

    private var mWidth = 0
    private var mAnimDuration = DEF_ANIM_DURATION
    private var mStrokeWidth = 0
    private var mCheckedColor = COLOR_CHECKED
    private var mUnCheckedColor = COLOR_UNCHECKED
    private var mFloorColor = COLOR_FLOOR_UNCHECKED
    private var mFloorUnCheckedColor = COLOR_FLOOR_UNCHECKED

    private var mChecked = false
    private var mTickDrawing = false
    private var mListener: OnCheckedChangeListener? = null

    init {
        init(attrs)
    }

    private fun init(attrs: AttributeSet?) {
        val ta = context.obtainStyledAttributes(attrs, R.styleable.SmoothCheckBox)
        val tickColor = ta.getColor(R.styleable.SmoothCheckBox_color_tick, COLOR_TICK)
        mAnimDuration = ta.getInt(R.styleable.SmoothCheckBox_duration, DEF_ANIM_DURATION)
        mFloorColor =
            ta.getColor(R.styleable.SmoothCheckBox_color_unchecked_stroke, COLOR_FLOOR_UNCHECKED)
        mCheckedColor = ta.getColor(R.styleable.SmoothCheckBox_color_checked, COLOR_CHECKED)
        mUnCheckedColor = ta.getColor(R.styleable.SmoothCheckBox_color_unchecked, COLOR_UNCHECKED)
        mStrokeWidth = ta.getDimensionPixelSize(R.styleable.SmoothCheckBox_stroke_width, dpToPx(0))
        ta.recycle()

        mFloorUnCheckedColor = mFloorColor

        mTickPaint = Paint(Paint.ANTI_ALIAS_FLAG).apply {
            style = Paint.Style.STROKE
            strokeCap = Paint.Cap.ROUND
            color = tickColor
        }

        mFloorPaint = Paint(Paint.ANTI_ALIAS_FLAG).apply {
            style = Paint.Style.FILL
            color = mFloorColor
        }

        mPaint = Paint(Paint.ANTI_ALIAS_FLAG).apply {
            style = Paint.Style.FILL
            color = mCheckedColor
        }

        mTickPath = Path()
        mCenterPoint = Point()
        mTickPoints = Array(3) { Point() }

        setOnClickListener {
            toggle()
            mTickDrawing = false
            mDrewDistance = 0f
            if (isChecked) {
                startCheckedAnimation()
            } else {
                startUnCheckedAnimation()
            }
        }
    }

    private fun dpToPx(dp: Int): Int = (dp * resources.displayMetrics.density).toInt()

    override fun onSaveInstanceState(): Parcelable {
        val bundle = Bundle()
        bundle.putParcelable(KEY_INSTANCE_STATE, super.onSaveInstanceState())
        bundle.putBoolean(KEY_INSTANCE_STATE, isChecked)
        return bundle
    }

    override fun onRestoreInstanceState(state: Parcelable?) {
        if (state is Bundle) {
            val bundle = state
            val isChecked = bundle.getBoolean(KEY_INSTANCE_STATE)
            setChecked(isChecked)
            super.onRestoreInstanceState(bundle.getParcelable(KEY_INSTANCE_STATE))
            return
        }
        super.onRestoreInstanceState(state)
    }

    override fun isChecked(): Boolean = mChecked

    override fun toggle() {
        setChecked(!isChecked())
    }

    override fun setChecked(checked: Boolean) {
        mChecked = checked
        reset()
        invalidate()
        mListener?.onCheckedChanged(this, mChecked)
    }

    /**
     * checked with animation
     * @param checked checked
     * @param animate change with animation
     */
    fun setChecked(checked: Boolean, animate: Boolean) {
        if (animate) {
            mTickDrawing = false
            mChecked = checked
            mDrewDistance = 0f
            if (checked) {
                startCheckedAnimation()
            } else {
                startUnCheckedAnimation()
            }
            mListener?.onCheckedChanged(this, mChecked)
        } else {
            setChecked(checked)
        }
    }

    private fun reset() {
        mTickDrawing = true
        mFloorScale = 1.0f
        mScaleVal = if (isChecked) 0f else 1.0f
        mFloorColor = if (isChecked) mCheckedColor else mFloorUnCheckedColor
        mDrewDistance = if (isChecked) (mLeftLineDistance + mRightLineDistance) else 0f
    }

    private fun measureSize(measureSpec: Int): Int {
        val defSize = dpToPx(DEF_DRAW_SIZE)
        val specSize = MeasureSpec.getSize(measureSpec)
        val specMode = MeasureSpec.getMode(measureSpec)

        return when (specMode) {
            MeasureSpec.UNSPECIFIED, MeasureSpec.AT_MOST -> minOf(defSize, specSize)
            MeasureSpec.EXACTLY -> specSize
            else -> defSize
        }
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        setMeasuredDimension(measureSize(widthMeasureSpec), measureSize(heightMeasureSpec))
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        mWidth = measuredWidth
        mStrokeWidth = (if (mStrokeWidth == 0) measuredWidth / 10 else mStrokeWidth).coerceIn(3, measuredWidth / 5)
        mCenterPoint.x = mWidth / 2
        mCenterPoint.y = measuredHeight / 2

        // 调整 tick 的坐标，放置在左边中间偏下位置
        mTickPoints[0].x = (mCenterPoint.x / 2)  // 水平左边中间
        mTickPoints[0].y = (mCenterPoint.y )  // 垂直偏下

        mTickPoints[1].x = mTickPoints[0].x + measuredWidth / 4  // 水平方向向右偏移
        mTickPoints[1].y = (mCenterPoint.y + measuredHeight / 6)  // 垂直偏下

        mTickPoints[2].x = mTickPoints[1].x + measuredWidth / 4  // 水平方向继续向右
        mTickPoints[2].y = (mCenterPoint.y - measuredHeight / 6)  // 稍微向上偏移

        // 计算线段的距离
        mLeftLineDistance = Math.sqrt(
            Math.pow((mTickPoints[1].x - mTickPoints[0].x).toDouble(), 2.0) +
                    Math.pow((mTickPoints[1].y - mTickPoints[0].y).toDouble(), 2.0)
        ).toFloat()

        mRightLineDistance = Math.sqrt(
            Math.pow((mTickPoints[2].x - mTickPoints[1].x).toDouble(), 2.0) +
                    Math.pow((mTickPoints[2].y - mTickPoints[1].y).toDouble(), 2.0)
        ).toFloat()

        mTickPaint.strokeWidth = mStrokeWidth.toFloat()
    }


    override fun onDraw(canvas: Canvas) {
        drawBorder(canvas)
        drawCenter(canvas)
        drawTick(canvas)
    }

    private fun drawCenter(canvas: Canvas) {
        mPaint.color = mUnCheckedColor
        val radius = (mCenterPoint.x - mStrokeWidth) * mScaleVal
        canvas.drawCircle(mCenterPoint.x.toFloat(), mCenterPoint.y.toFloat(), radius, mPaint)
    }

    private fun drawBorder(canvas: Canvas) {
        mFloorPaint.color = mFloorColor
        val radius = mCenterPoint.x
        canvas.drawCircle(
            mCenterPoint.x.toFloat(),
            mCenterPoint.y.toFloat(),
            radius * mFloorScale,
            mFloorPaint
        )
    }

    private fun drawTick(canvas: Canvas) {
        if (mTickDrawing && isChecked) {
            drawTickPath(canvas)
        }
    }

    private fun drawTickPath(canvas: Canvas) {
        mTickPath.reset()
        mTickPath.moveTo(mTickPoints[0].x.toFloat(), mTickPoints[0].y.toFloat())
        mTickPath.lineTo(mTickPoints[1].x.toFloat(), mTickPoints[1].y.toFloat())
        mTickPath.lineTo(mTickPoints[2].x.toFloat(), mTickPoints[2].y.toFloat())
        canvas.drawPath(mTickPath, mTickPaint)
    }

    private fun startCheckedAnimation() {
        ValueAnimator.ofFloat(1.0f, 0f).apply {
            duration = (mAnimDuration / 3 * 2).toLong()
            interpolator = LinearInterpolator()
            addUpdateListener {
                mScaleVal = it.animatedValue as Float
                mFloorColor = getGradientColor(mUnCheckedColor, mCheckedColor, 1 - mScaleVal)
                postInvalidate()
            }
            start()
        }

        ValueAnimator.ofFloat(1.0f, 0.8f, 1.0f).apply {
            duration = mAnimDuration.toLong()
            interpolator = LinearInterpolator()
            addUpdateListener {
                mFloorScale = it.animatedValue as Float
                postInvalidate()
            }
            start()
        }

        drawTickDelayed()
    }

    private fun startUnCheckedAnimation() {
        ValueAnimator.ofFloat(0f, 1.0f).apply {
            duration = mAnimDuration.toLong()
            interpolator = LinearInterpolator()
            addUpdateListener {
                mScaleVal = it.animatedValue as Float
                mFloorColor = getGradientColor(mCheckedColor, mFloorUnCheckedColor, mScaleVal)
                postInvalidate()
            }
            start()
        }

        ValueAnimator.ofFloat(1.0f, 0.8f, 1.0f).apply {
            duration = mAnimDuration.toLong()
            interpolator = LinearInterpolator()
            addUpdateListener {
                mFloorScale = it.animatedValue as Float
                postInvalidate()
            }
            start()
        }
    }

    private fun drawTickDelayed() {
        postDelayed({
            mTickDrawing = true
            postInvalidate()
        }, mAnimDuration.toLong())
    }

    private fun getGradientColor(startColor: Int, endColor: Int, percent: Float): Int {
        val startA = Color.alpha(startColor)
        val startR = Color.red(startColor)
        val startG = Color.green(startColor)
        val startB = Color.blue(startColor)

        val endA = Color.alpha(endColor)
        val endR = Color.red(endColor)
        val endG = Color.green(endColor)
        val endB = Color.blue(endColor)

        val currentA = (startA * (1 - percent) + endA * percent).toInt()
        val currentR = (startR * (1 - percent) + endR * percent).toInt()
        val currentG = (startG * (1 - percent) + endG * percent).toInt()
        val currentB = (startB * (1 - percent) + endB * percent).toInt()

        return Color.argb(currentA, currentR, currentG, currentB)
    }

    fun setOnCheckedChangeListener(listener: OnCheckedChangeListener?) {
        mListener = listener
    }

    interface OnCheckedChangeListener {
        fun onCheckedChanged(checkBox: SmoothCheckBox, isChecked: Boolean)
    }
}
