package com.kaixed.kchat.utils

import okhttp3.*
import java.io.File
import java.io.IOException
import java.io.RandomAccessFile

class DownloadManager(
    private val url: String,
    private val destinationFile: File,
    private val callback: DownloadCallback
) {

    private val client = OkHttpClient()
    private var downloadedBytes: Long = 0L
    private var totalBytes: Long = 0L

    init {
        // 如果目标文件已存在，则获取已下载的字节数
        if (destinationFile.exists()) {
            downloadedBytes = destinationFile.length()
        }
    }

    // 开始下载文件
    fun startDownload() {
        val request = Request.Builder()
            .url(url)
            .header("Range", "bytes=$downloadedBytes-") // 请求从已下载的字节数继续下载
            .build()

        client.newCall(request).enqueue(object : Callback {
            override fun onResponse(call: Call, response: Response) {
                if (response.isSuccessful) {
                    // 获取文件总大小
                    val contentRange = response.header("Content-Range")
                    if (contentRange != null) {
                        totalBytes = contentRange.split("/")[1].toLong()
                    }

                    // 获取响应体的输入流
                    val body = response.body ?: return

                    // 使用 RandomAccessFile 写入文件
                    val raf = RandomAccessFile(destinationFile, "rw")
                    raf.seek(downloadedBytes)

                    val inputStream = body.byteStream()
                    val buffer = ByteArray(8192)
                    var bytesRead: Int

                    try {
                        // 开始下载并写入文件
                        while (inputStream.read(buffer).also { bytesRead = it } != -1) {
                            raf.write(buffer, 0, bytesRead)
                            downloadedBytes += bytesRead

                            // 计算下载进度
                            val progress = (downloadedBytes * 100) / totalBytes

                            // 回调进度更新
                            callback.onProgress(progress)
                        }

                        // 下载完成
                        callback.onDownloadComplete()

                    } catch (e: IOException) {
                        // 发生错误时回调
                        callback.onError(e.message ?: "Unknown error")
                    } finally {
                        raf.close()
                        inputStream.close()
                        body.close()
                    }
                } else {
                    callback.onError("Failed to download: ${response.code}")
                }
            }

            override fun onFailure(call: Call, e: IOException) {
                // 网络请求失败时回调
                callback.onError("Download failed: ${e.message}")
            }
        })
    }

    // 取消下载
    fun cancelDownload() {
        // 可以扩展实现下载取消的功能，关闭请求等
    }

    // 下载进度回调接口
    interface DownloadCallback {
        fun onProgress(progress: Long)  // 下载进度（百分比）
        fun onDownloadComplete()  // 下载完成
        fun onError(errorMessage: String)  // 错误信息
    }
}
