package com.kaixed.kchat.utils

import android.app.Activity
import android.content.res.Resources
import android.graphics.Rect
import android.os.Build
import android.view.View
import android.view.ViewTreeObserver
import android.view.WindowInsets
import android.view.WindowInsetsAnimation
import android.view.WindowManager
import android.widget.FrameLayout
import androidx.annotation.RequiresApi
import androidx.core.view.ViewCompat
import androidx.core.view.WindowInsetsCompat

object KeyboardUtils {

    private var sDecorViewInvisibleHeightPre: Int = 0
    private var onGlobalLayoutListener: ViewTreeObserver.OnGlobalLayoutListener? = null
    private var mNavHeight: Int = 0
    private var sDecorViewDelta: Int = 0

    private fun getDecorViewInvisibleHeight(activity: Activity): Int {
        val decorView = activity.window.decorView ?: return sDecorViewInvisibleHeightPre
        val outRect = Rect()
        decorView.getWindowVisibleDisplayFrame(outRect)

        val delta = Math.abs(decorView.bottom - outRect.bottom)
        if (delta <= mNavHeight) {
            sDecorViewDelta = delta
            return 0
        }
        return delta - sDecorViewDelta
    }

    fun registerKeyboardHeightListener(activity: Activity, listener: KeyboardHeightListener) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            invokeAbove31(activity, listener)
        } else {
            invokeBelow31(activity, listener)
        }
    }

    @RequiresApi(Build.VERSION_CODES.R)
    private fun invokeAbove31(activity: Activity, listener: KeyboardHeightListener) {
        activity.window.decorView.setWindowInsetsAnimationCallback(object : WindowInsetsAnimation.Callback(DISPATCH_MODE_STOP) {
            override fun onProgress(windowInsets: WindowInsets, animations: List<WindowInsetsAnimation>): WindowInsets {
                val imeHeight = windowInsets.getInsets(WindowInsetsCompat.Type.ime()).bottom
                val navHeight = windowInsets.getInsets(WindowInsetsCompat.Type.navigationBars()).bottom
                val hasNavigationBar = windowInsets.isVisible(WindowInsetsCompat.Type.navigationBars()) &&
                        windowInsets.getInsets(WindowInsetsCompat.Type.navigationBars()).bottom > 0

                listener.onKeyboardHeightChanged(if (hasNavigationBar) Math.max(imeHeight - navHeight, 0) else imeHeight)
                return windowInsets
            }
        })
    }

    private fun invokeBelow31(activity: Activity, listener: KeyboardHeightListener) {
        val flags = activity.window.attributes.flags

        if (flags and WindowManager.LayoutParams.FLAG_LAYOUT_NO_LIMITS != 0) {
            activity.window.clearFlags(WindowManager.LayoutParams.FLAG_LAYOUT_NO_LIMITS)
        }

        val contentView = activity.findViewById<FrameLayout>(android.R.id.content)
        sDecorViewInvisibleHeightPre = getDecorViewInvisibleHeight(activity)

        onGlobalLayoutListener = object : ViewTreeObserver.OnGlobalLayoutListener {
            override fun onGlobalLayout() {
                val height = getDecorViewInvisibleHeight(activity)
                if (sDecorViewInvisibleHeightPre != height) {
                    listener.onKeyboardHeightChanged(height)
                    sDecorViewInvisibleHeightPre = height
                }
            }
        }

        getNavigationBarHeight(activity) { height, hasNav ->
            mNavHeight = height
            contentView.viewTreeObserver.addOnGlobalLayoutListener(onGlobalLayoutListener)
        }
    }

    fun unregisterKeyboardHeightListener(activity: Activity) {
        onGlobalLayoutListener = null
        val contentView = activity.window.decorView.findViewById<View>(android.R.id.content) ?: return
        contentView.viewTreeObserver.removeOnGlobalLayoutListener(onGlobalLayoutListener)
    }

    private fun getNavBarHeight(): Int {
        val res = Resources.getSystem()
        val resourceId = res.getIdentifier("navigation_bar_height", "dimen", "android")
        return if (resourceId != 0) {
            res.getDimensionPixelSize(resourceId)
        } else {
            0
        }
    }

    private fun getNavigationBarHeight(activity: Activity, callback: (height: Int, hasNav: Boolean) -> Unit) {
        val view = activity.window.decorView
        val attachedToWindow = view.isAttachedToWindow

        if (attachedToWindow) {
            val windowInsets = ViewCompat.getRootWindowInsets(view)
            val height = windowInsets?.getInsets(WindowInsetsCompat.Type.navigationBars())?.bottom ?: 0
            val hasNavigationBar = windowInsets?.isVisible(WindowInsetsCompat.Type.navigationBars()) == true &&
                    windowInsets.getInsets(WindowInsetsCompat.Type.navigationBars()).bottom > 0

            if (height > 0) {
                callback(height, hasNavigationBar)
            } else {
                callback(getNavBarHeight(), hasNavigationBar)
            }
        } else {
            view.addOnAttachStateChangeListener(object : View.OnAttachStateChangeListener {
                override fun onViewAttachedToWindow(v: View) {
                    val windowInsets = ViewCompat.getRootWindowInsets(v)
                    val height = windowInsets?.getInsets(WindowInsetsCompat.Type.navigationBars())?.bottom ?: 0
                    val hasNavigationBar = windowInsets?.isVisible(WindowInsetsCompat.Type.navigationBars()) == true &&
                            windowInsets.getInsets(WindowInsetsCompat.Type.navigationBars()).bottom > 0

                    if (height > 0) {
                        callback(height, hasNavigationBar)
                    } else {
                        callback(getNavBarHeight(), hasNavigationBar)
                    }
                }

                override fun onViewDetachedFromWindow(v: View) {}
            })
        }
    }


    interface KeyboardHeightListener {
        fun onKeyboardHeightChanged(height: Int)
    }

    interface NavigationBarCallback {
        fun onHeight(height: Int, hasNav: Boolean)
    }
}
