package com.kaixed.core.network

import com.kaixed.core.common.utils.NetworkInterface
import com.kaixed.core.network.service.AuthApiService
import com.kaixed.core.network.service.FileApiService
import com.kaixed.core.network.service.FriendApiService
import com.kaixed.core.network.service.UserApiService
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import java.util.concurrent.TimeUnit

/**
 * @Author: kaixed
 * @Date: 2024/11/15 11:05
 */
object RetrofitClient {

    private const val BASE_URL = "${NetworkInterface.SERVER_URL}/"

    // 添加日志拦截器
    private val loggingInterceptor = HttpLoggingInterceptor().apply {
        level = HttpLoggingInterceptor.Level.BODY
    }

    // 创建一个不包含 Token 拦截器的 OkHttpClient，专门用于授权 API
    private val authClient = OkHttpClient.Builder()
        .addInterceptor(loggingInterceptor)
        .pingInterval(15, TimeUnit.SECONDS)
        .build()

    // 创建单独的 Retrofit 实例（专门用于授权 API）
    private val authRetrofit: Retrofit by lazy {
        Retrofit.Builder()
            .baseUrl(BASE_URL)
            .addConverterFactory(GsonConverterFactory.create())
            .client(authClient)  // 使用不包含 Token 拦截器的 OkHttpClient
            .build()
    }

    // 授权 API 服务实例，使用不包含 Token 拦截器的 Retrofit 实例
    val authApiService: AuthApiService by lazy {
        authRetrofit.create(AuthApiService::class.java)
    }

    // 创建一个 OkHttpClient，包含 Token 拦截器
    private val client = OkHttpClient.Builder()
        .addInterceptor(loggingInterceptor)
        .pingInterval(15, TimeUnit.SECONDS)
//        .addInterceptor(SignInterceptor()) // 签名拦截器
//        .addInterceptor(TokenRefreshInterceptor())  // Token 拦截器
        .build()

    // 创建 Retrofit 实例（用于普通 API 调用）
    private val retrofit: Retrofit by lazy {
        Retrofit.Builder()
            .baseUrl(BASE_URL)
            .addConverterFactory(GsonConverterFactory.create())
            .client(client)  // 使用包含 Token 拦截器的 OkHttpClient
            .build()
    }

    // API 服务实例
    val userApiService: UserApiService by lazy {
        retrofit.create(UserApiService::class.java)
    }

    val friendApiService: FriendApiService by lazy {
        retrofit.create(FriendApiService::class.java)
    }

    val fileApiService: FileApiService by lazy {
        retrofit.create(FileApiService::class.java)
    }
}
