package com.kaixed.core.network.interceptor

import com.kaixed.core.common.utils.Constants
import okhttp3.FormBody
import okhttp3.Interceptor
import okhttp3.Request
import okhttp3.Response
import okio.IOException
import java.security.MessageDigest
import java.util.*

class SignInterceptor : Interceptor {

    @Throws(IOException::class)
    override fun intercept(chain: Interceptor.Chain): Response {
        val request = chain.request()

        // 获取当前时间戳
        val timestamp = System.currentTimeMillis().toString()

        // 生成随机 nonce
        val nonce = UUID.randomUUID().toString()

        // 获取请求参数
        val params = getRequestParams(request)

        // 生成签名
        val sign = generateServerSign(params, timestamp, nonce)

        // 构造新的请求，添加请求头
        val newRequest = request.newBuilder()
            .addHeader("sign", sign)
            .addHeader("timestamp", timestamp)
            .addHeader("nonce", nonce)
            .build()

        // 继续请求
        return chain.proceed(newRequest)
    }

    // 获取请求的参数，处理 POST 请求的 Map 格式数据，并确保按字典顺序排序
    private fun getRequestParams(request: Request): Map<String, String> {
        val params = TreeMap<String, String>()

        when (request.method) {
            "GET" -> {
                request.url.queryParameterNames.forEach { name ->
                    params[name] = request.url.queryParameter(name) ?: ""
                }
            }

            "POST" -> {
                val body = request.body
                if (body is FormBody) {
                    // 获取所有参数，并确保按字典顺序排序
                    for (i in 0 until body.size) {
                        params[body.name(i)] = body.value(i)
                    }
                }
            }

            else -> {
                // 其他请求方法不处理
            }
        }
        return params
    }

    // 生成服务端签名
    private fun generateServerSign(
        sortedParams: Map<String, String>,
        timestamp: String,
        nonce: String,
    ): String {
        // 拼接参数
        val sb = StringBuilder()
        sortedParams.forEach { (key, value) ->
            sb.append("$key=$value&")
        }
        sb.append("timestamp=$timestamp&")
        sb.append("nonce=$nonce&")
        sb.append("secretKey=${Constants.Sign.SECRET_KEY}")

        // 使用 SHA-256 进行加密
        return hashWithSHA256(sb.toString())
    }


    // 使用 SHA-256 进行哈希加密
    private fun hashWithSHA256(input: String): String {
        val digest = MessageDigest.getInstance("SHA-256")
        val hashBytes = digest.digest(input.toByteArray(Charsets.UTF_8))
        val hexString = StringBuilder()
        for (b in hashBytes) {
            val hex = Integer.toHexString(0xff and b.toInt())
            if (hex.length == 1) hexString.append('0')
            hexString.append(hex)
        }
        return hexString.toString()
    }
}
