package com.kaixed.core.network.message

import android.util.Log
import com.kaixed.core.common.utils.ConstantsUtils.getUsername
import com.kaixed.core.network.OkhttpHelper
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.SupervisorJob
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.receiveAsFlow
import kotlinx.coroutines.launch
import okhttp3.Request
import okhttp3.Response
import okhttp3.WebSocket
import okhttp3.WebSocketListener

class OkHttpWebSocketClient : WebSocketClient {

    companion object {
        const val TAG = "WebSocketService"
        const val WEBSOCKET_CLOSE_CODE = 1000
        const val MAX_RECONNECT_ATTEMPTS = 5
    }

    private var reconnectAttempts = 0

    private val _messages = Channel<String>(capacity = 100)

    private val scope: CoroutineScope = CoroutineScope(Dispatchers.IO + SupervisorJob())
    private val url: String = "$WEBSOCKET_SERVER_URL$WEBSOCKET${getUsername()}"
    private lateinit var webSocket: WebSocket
    private var isConnected = false

    override fun connect() {
        val request = Request.Builder()
            .url(url).build()
        val listener = EchoWebSocketListener()
        val client = OkhttpHelper.getInstance()
        webSocket = client.newWebSocket(request, listener)
    }

    override fun send(message: String, msgLocalId: Long): Boolean {
        return if (isConnected) {
            webSocket.send(message)
        } else {
            false
        }
    }

    override fun close(code: Int, reason: String) {
        webSocket.close(code, reason)
        isConnected = false
    }

    override fun isConnected(): Boolean = isConnected

    private fun reconnect() {
        if (reconnectAttempts >= MAX_RECONNECT_ATTEMPTS) {
            Log.e("WebSocket", "达到最大重连次数")
            return
        }
        reconnectAttempts++
        scope.launch {
            val delayMs = minOf(reconnectAttempts * 1000L, 30000L) // 指数退避，最大 30 秒
            delay(delayMs)
            connect()
        }
    }

    override fun observeMessages(): Flow<String> = _messages.receiveAsFlow()

    inner class EchoWebSocketListener : WebSocketListener() {
        override fun onOpen(webSocket: WebSocket, response: Response) {
            Log.d(TAG, "WebSocket Opened")
            isConnected = true
            reconnectAttempts = 0
        }

        override fun onMessage(webSocket: WebSocket, text: String) {
            scope.launch { _messages.send(text) }
        }

        override fun onClosing(webSocket: WebSocket, code: Int, reason: String) {
            isConnected = false
            webSocket.close(WEBSOCKET_CLOSE_CODE, null)
            Log.d(TAG, "WebSocket closing: $reason")
            connect()
        }

        override fun onFailure(webSocket: WebSocket, t: Throwable, response: Response?) {
            isConnected = false
            Log.d(TAG, "WebSocket closing: ${t.cause}")
            reconnect()
        }
    }
}